// Spectra
// Copyright (C) 1998, Earl F. Glynn, Overland Park, KS.
// May be copied freely for non-commercial use.
// E-Mail:  EarlGlynn@att.net

unit ScreenSpectra;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  ExtCtrls, StdCtrls, Spin, ComCtrls, Buttons, TeEngine, Series, TeeProcs,
  Chart, ExtDlgs, BSpectrum;

type
  TFormSpectra = class(TForm)
    PageControl1: TPageControl;
    TabSheetVisibleLight: TTabSheet;
    TabSheetRGBGraph: TTabSheet;
    LabelWavelength: TLabel;
    TabAtomicSpectra: TTabSheet;
    ChartRGBWavelength: TChart;
    SeriesRed: TLineSeries;
    SeriesGreen: TLineSeries;
    SeriesBlue: TLineSeries;
    LabelSourceCredit: TLabel;
    I: TCheckBox;
    D: TSpinEdit;
    LabelInterval: TLabel;
    SeriesIntensity: TLineSeries;
    SeriesY: TLineSeries;
    CheckBoxIntensity: TCheckBox;
    CheckBoxY: TCheckBox;
    ButtonPrintGraph: TButton;
    P: TCheckBox;
    LabelInfo2A: TLabel;
    LabelLambda: TLabel;
    LabelInfo3: TLabel;
    LabelInfo4A: TLabel;
    LabelInfo4B: TLabel;
    LabelHiddenComment1: TLabel;
    LabelHiddenComment2: TLabel;
    LabelHiddenComment3: TLabel;
    LabelInfo5: TLabel;
    LabelInfo6: TLabel;
    LabelInfo7: TLabel;
    LabelInfo8: TLabel;
    LabelInfo9: TLabel;
    LabelInfo10: TLabel;
    LabelInfo1: TLabel;
    LabelInfo2B: TLabel;
    LabelInfoSource1: TLabel;
    LabelInfoSource2: TLabel;
    LabelInfo11: TLabel;
    LabelInfo12: TLabel;
    LabelInfo13: TLabel;
    LabelInfo14: TLabel;
    SavePictureDialogSpectrum: TSavePictureDialog;
    Label256ColorComment: TLabel;
    LabelRGB: TLabel;
    S: TBSpectrum;
    Label1: TLabel;
    E: TBSpectrum;
    A: TBSpectrum;
    Label2: TLabel;
    Label3: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure CheckBoxIntensityClick(Sender: TObject);
    procedure CheckBoxYClick(Sender: TObject);
    procedure ButtonPrintGraphClick(Sender: TObject);
    procedure TabSheetVisibleLightMouseMove(Sender: TObject;
      Shift: TShiftState; X, Y: Integer);
    procedure SMouseMove(Shift: TShiftState; X, Y: Integer; R, G,
      B: Byte);
    procedure PClick(Sender: TObject);
    procedure DKeyPress(Sender: TObject; var Key: Char);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  FormSpectra: TFormSpectra;

implementation
{$R *.DFM}

  USES
    Printers,SpectraLibrary;

{$IFDEF VER100}          // Needed in Delphi 3
  TYPE                   // for Delphi 4 compatibility
    LongWord = INTEGER;
{$ENDIF}

  // Based on posting to borland.public.delphi.winapi by Rodney E Geraghty, 8/8/97.
  // Used to print bitmap on any Windows printer.}
  PROCEDURE PrintBitmap(Canvas:  TCanvas; DestRect:  TRect;  Bitmap:  TBitmap);
    VAR
      BitmapHeader:  pBitmapInfo;
      BitmapImage :  POINTER;
      HeaderSize  :  LongWord;   // New Delphi 4 Type
      ImageSize   :  LongWord;   // New Delphi 4 Type
  BEGIN
    GetDIBSizes(Bitmap.Handle, HeaderSize, ImageSize);
    GetMem(BitmapHeader, HeaderSize);
    GetMem(BitmapImage,  ImageSize);
    TRY
      GetDIB(Bitmap.Handle, Bitmap.Palette, BitmapHeader^, BitmapImage^);
      StretchDIBits(Canvas.Handle,
                    DestRect.Left, DestRect.Top,     {Destination Origin}
                    DestRect.Right  - DestRect.Left, {Destination Width}
                    DestRect.Bottom - DestRect.Top,  {Destination Height}
                    0, 0,                            {Source Origin}
                    Bitmap.Width, Bitmap.Height,     {Source Width & Height}
                    BitmapImage,
                    TBitmapInfo(BitmapHeader^),
                    DIB_RGB_COLORS,
                    SRCCOPY)
    FINALLY
      FreeMem(BitmapHeader);
      FreeMem(BitmapImage)
    END
  END {DrawBitmap};


  FUNCTION  GetPixelDepth:  INTEGER;
    VAR
      DeviceContext:  hDC;
  BEGIN
    // Get the screen's DC since memory DCs are not reliable
    DeviceContext := GetDC(0);

    TRY
      RESULT := GetDeviceCaps(DeviceContext, BITSPIXEL)
    FINALLY
      // Give back the screen DC
      ReleaseDC(0, DeviceContext)
    END;
  END {GetPixelDepth};


procedure TFormSpectra.FormCreate(Sender: TObject);
  VAR
    R,G,B     :  BYTE;
    Wavelength:  INTEGER;
begin
  Label256ColorComment.Visible := (GetPixelDepth < 16);

  FOR Wavelength := WavelengthMinimum TO WavelengthMaximum DO
  BEGIN
    WavelengthToRGB(Wavelength, R,G,B);
    SeriesRed.AddXY(Wavelength, R, '', clTeeColor);
    SeriesGreen.AddXY(Wavelength, G, '', clTeeColor);
    SeriesBlue.AddXY(Wavelength, B, '', clTeeColor);
    SeriesIntensity.AddXY(Wavelength, ROUND( (R+G+B)/3 ), '', clTeeColor);
    SeriesY.AddXY(Wavelength, ROUND(0.299*R + 0.587*G + 0.114*B), '', clTeeColor)
  END
end;


procedure TFormSpectra.CheckBoxIntensityClick(Sender: TObject);
begin
  SeriesIntensity.Active := CheckBoxIntensity.Checked
end;

procedure TFormSpectra.CheckBoxYClick(Sender: TObject);
begin
  SeriesY.Active := CheckBoxY.Checked
end;

procedure TFormSpectra.ButtonPrintGraphClick(Sender: TObject);
begin
  ChartRGBWavelength.PrintLandscape
end;


procedure TFormSpectra.TabSheetVisibleLightMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
begin
  LabelRGB.Caption := ''
end;

procedure TFormSpectra.SMouseMove(Shift: TShiftState; X,
  Y: Integer; R, G, B: Byte);
begin
  LabelRGB.Caption :=   'R=' + IntToStr(R)+
                      '  G=' + IntToStr(G)+
                      '  B=' + IntToSTr(B);

end;





procedure TFormSpectra.PClick(Sender: TObject);
begin
 S.IntervalDelta:=D.Value;
 E.IntervalDelta:=D.Value;
 A.IntervalDelta:=D.Value;

 S.PhotoPlateOrder:=P.Checked;
 E.PhotoPlateOrder:=P.Checked;
 A.PhotoPlateOrder:=P.Checked;

 S.Interval:=I.Checked;
 E.Interval:=I.Checked;
 A.Interval:=I.Checked;

 S.Invalidate;
 E.Invalidate;
 A.Invalidate;
end;

procedure TFormSpectra.DKeyPress(Sender: TObject; var Key: Char);
begin
 Key:=#0;
end;

end.
